#!/usr/bin/perl -w

use Fcntl 'O_RDONLY';
use Getopt::Long;
use File::Spec;
use File::Basename;
use strict;
use warnings;
use Scalar::Util qw(looks_like_number);
use List::Util qw(max min sum);
use GD::Graph::bars;

$| = 1;

# Reads a phenotype file of tab seperated values and performs checks on
# them, printing out a report at the end detailing any errors found.
#
#
#
#
#
# usage perl phenotype_qc.pl -f filename [-v]
#
# V3.0 changed to use tied arrays rather than load data set as hash into memory
# v3.1 Added Sample fingerprint
# v4.0 removed tied arrays and replaced with back tick call to cut
# v4.1 updated cut to use vairable number of column chunks to increase speed

# get today's date
my $time = localtime;

my $delimiter = '\t';
my $filepath;
my $today;
my $name;
my $path;
my $suffix;
my @suffixlist = (".txt", ".tab", ".tsv");
my $file;
my $verbose_flag = 0;
my $help = 0;
my $missing = 'NA';
my $headerfile = '';
my $idcolumn = 0; #default column for sample ids
my $column;       #column for a single phenotype  
my %columns;      #column for multiple phenotypes
my $columnfile;   #file to provide multiple phenotypes for %columns
my %headers;
my $plots = 0;
my $chunk = 25;

GetOptions
 (
 "f|file=s"      => \$filepath,      #file to be checked
 "v|verbose"     => \$verbose_flag,  #specify verbose output (prints to logfile)
 "l|lookup=s",   => \$headerfile,    #specify a header file mapping in case they are not human readable
 "i|idcolumn=s", => \$idcolumn,      #specify the id containing column, if it's not the first one
 "c|column=s"    => \$column,        #check a single phenotype in the file
 "s|columnfile=s"=> \$columnfile,    #specify a file containing many phenotypes to check
 "m|missing=s"   => \$missing,       #specify the missing value identifier, if not NA
 "k|chunk=s"     => \$chunk,         #specify how may columns to extract in each chunk
 "p|plots"       => \$plots,         #specify you want the graphs plotted
 "h|help"        => \$help           #print the help message
 );

if ($help)
 {
 usage();
 exit;
 }

if (!$filepath)
 {
 usage();
 exit;
 }

print "       \n\nPhenotype QC\n";
print "         W.Rayner\n";
print "        Version 4.1\n\n\n";
print "Options:\n";
print "File:              $filepath\n";
if ($headerfile)
 {
 print "Header File:       $headerfile\n";
 }
print "Missing Value:     $missing\n"; 
print "Extraction size:   $chunk\n";
print "Plots:             ";
if ($plots)
 {
 print "Yes\n";
 }
else
 {
 print "No\n";
 }
if ($column) 
 {
 print "Column to extract: $column\n";
 }
if ($columnfile)
 {
 print "Column file:       $columnfile\n";
 }
 
print "\n\n"; 
 
if ($time=~/\w+?\s(\w+?)\s+(\d+)\s.+?\s(\d+)/)
 {
 $today = sprintf("%02d", $2).$1.sprintf("%4d", $3);
 }

if ($headerfile)
 {
 %headers = get_headers($headerfile);
 }

if ($columnfile)
 {
 %columns = get_columns($columnfile);
 }
elsif ($column)
 {
 $columns{$column} = 1;
 }
else
 {
 %columns = all_columns($filepath);
 }

my @rt = keys %columns;
my $crt = @rt;
print "Found $crt columns to extract\n";

($name, $path, $suffix) = fileparse($filepath,@suffixlist);

my $outputfilename = 'Summaryv1.'.$name.'_'.$today.'.txt';
my $logfilename = 'Logv1.'.$name.'.'.$today.'.txt';

# these files are appended to, check at start if they exist, if so rename
my $fpfile = 'FPv1.'.$name.'.'.$today.'.txt';   #fingerprint of the phenotypes (columns)
my $fpsfile = 'FPSv1.'.$name.'.'.$today.'.txt'; #fingerprint of the samples (rows)
$fpfile = check_file($fpfile);
$fpsfile = check_file($fpsfile);
$outputfilename = check_file($outputfilename);
$logfilename = check_file($logfilename);

open OUTPUT, ">$outputfilename" or die "Unable to open $outputfilename\n";
open LOGFILE, ">$logfilename" or die "unable to open logfile\n";
open STDERR, ">error.log" or die "unable to open error log file\n";

print OUTPUT "\t\tPhenotype QC report\n\n";
print OUTPUT "\tInput file:\t\t\t$filepath\n";
print OUTPUT "\tDate created:\t\t\t$today\n";
print OUTPUT "\tMissing value identifier:\t$missing\n\n";
#print OUTPUT "\tCreated William Rayner 2016 version 4.0\n";
#print OUTPUT "Values for each item are only shown if there are less than 50 items\n";
print OUTPUT "The verbose flag (set using -v on the command line) will result in \n";
print OUTPUT "all values will be written to the log file\n";

if ($verbose_flag)
 {
 print OUTPUT "Verbose flag is set\n\n";
 }
print OUTPUT "------------------------------------------------------------------\n\n";

#summarise columns and generate FP summary
my @dataColumns = get_data_columns($filepath, $idcolumn, \%columns);
extract_data($filepath, $missing, $fpfile, $chunk, \@dataColumns);
close OUTPUT;

#summarise rows and generate FPS summary
my %column_numbers = get_column_numbers($filepath, \%columns);
$column_numbers{$idcolumn} = 1;
get_fps($filepath, $idcolumn, $missing, $fpsfile, \%column_numbers);


sub get_column_numbers
 {
 my $file = $_[0];
 my %cols = %{$_[1]};
 my %coln;
 
 open IN, "$file" or die $!;
 my $headerline = <IN>;
 close IN;
 chomp $headerline;
 my @temp = split(/\t/, $headerline); 
 
 for (my $i = 0; $i <= $#temp; $i++)
  {
  if ($cols{$temp[$i]} or $cols{$headers{$temp[$i]}})
   {
   $coln{$i} = 1;
   }
  else
   {
   $coln{$i} = 0;
   }
  }
  
 return %coln;
 }

sub all_columns
 {
 my $file = $_[0];
 my %allCols;
 open IN, "$file" or die $!;
 my $header = <IN>;
 close IN;
 chomp $header;
 my @columns = split(/\t/, $header, -1);
 $allCols{$_}++ for (@columns);
 return %allCols;
 }

sub get_columns
 {
 my $file = $_[0];
 my %cols;
 open IN, "$file" or die $!;
 while (<IN>)
  {
  chomp;
  my @temp = split/\t/;
  $cols{$temp[0]} = 1;
  }
 return %cols
 }

sub check_file
 {
 my $filename = $_[0];
 my $newfilename = $filename;
 if (-e $filename)
  {
  if ($filename =~ /(.*?v)(\d+)(\..*)$/)
   {
   my $num = $2;
   $num++;
   $newfilename = $1.$num.$3;
   $newfilename = check_file($newfilename);
   }
  else
   {
   print "ERROR: parsing filename failed\n";
   }
  }
 else
  {
  $newfilename = $filename;
  }
 return $newfilename;
 }

sub get_fps
 {
 my $filename = $_[0];
 my $sampleidcolumn = $_[1];
 my $missing = $_[2];
 my $fpsfile = $_[3];
 my %cols = %{$_[4]};
 
 print "Reading $filename\n";
 open IN, "$filename";
 while (<IN>)
  {
  my @data;
  chomp;
  my @temp = split/\t/;
  for (my $i = 0; $i <= $#temp; $i++)
   {
   if ($cols{$i})
    {
    push (@data, $temp[$i]);
    }
   }
  my $type = get_type(\@data, $missing, $fpsfile);
  }
 close IN; 
 } 

sub get_data_columns
 {
 my $filename = $_[0];
 my $sampleidcolumn = $_[1];
 my %cols = %{$_[2]};
 my @indexes;
 my $k = 0;
 
 print "Reading $filename\n";
 open IN, "$filename";
 my $line = <IN>;
 chomp $line;
 close IN;
 my @elements = split(/\t/, $line, -1);
 #print "Elements $#elements $elements[$#elements]\n";
 
 for (my $j = 0; $j <= $#elements; $j++)
  {
  if ($cols{$elements[$j]} or $cols{$headers{$elements[$j]}})
   {
   if ($j != $sampleidcolumn) #don't try and summarise the sample ids idcolumn
    {
    my $colidx = $j+1;
    $indexes[$k] = $colidx;
    $k++;
    }
   else
    {
    print OUTPUT "\nSkipping Sample id column $j: $elements[$j]\n";
    print "Skipping $elements[$j] as idcolumn containing sample id\n";
    }
   } 
  else
   {
   if ($headerfile)
    {
    if ($headers{$elements[$j]})
     {
     #print "Skipping $headers{$elements[$j]}\n";
     }
    else
     {
     #print "Skipping $elements[$j]\n";
     }
    }
   else
    {
    #print "Skipping $elements[$j]\n";
    }
   }
  }
  
 my $total = @indexes;
 print "Found $total columns in $filename to check\n";
 
 return @indexes;
 }
 
sub extract_data 
 {
 my $filename = $_[0];
 my $missing = $_[1];
 my $fpfile = $_[2];
 my $chunk = $_[3];
 my @indexes = @{$_[4]};
 
 my $colidxs = $indexes[0];
 
 if (0 == $#indexes) # 1 column only run without further processing
  {
  print "Extracting columns $colidxs\n";
  run_cut($filename, $colidxs, $missing, $fpfile, 0);
  }
 else
  {
  for (my $i = 1; $i <= $#indexes; $i++)
   {
   if ($i == $#indexes) 
    {
    $colidxs = $colidxs.','.$indexes[$i];
    print "Extracting columns $colidxs\n";
    run_cut($filename, $colidxs, $missing, $fpfile, $i);
    }
   elsif ($i % $chunk == 1 and $i != 1)
    {
    print "Extracting columns $colidxs\n";
    run_cut($filename, $colidxs, $missing, $fpfile, $i);
    $colidxs = $indexes[$i];
    } 
   else
    {
    $colidxs = $colidxs.','.$indexes[$i];
    }
   }
  } 
 }

sub run_cut
 {
 my $filename = $_[0];
 my $cols = $_[1];
 my $missing = $_[2];
 my $fpfile = $_[3];
 my $i = $_[4];
 my @final;
 my $count;
 
 #get data from cut
 my $dataline = `cut -f $cols $filename`;
 
 #process into lines
 my @data = split(/\n/, $dataline);
 
 #split lines and create array
 for (my $k = 0; $k <= $#data; $k++)
  {
  my @temp = split(/\t/, $data[$k]);
  $count = @temp;
  for (my $j = 0; $j <= $#temp; $j++)
   {
   $final[$j][$k] = $temp[$j];
   }
  }
    
 for (my $k = 0; $k < $count; $k++)
  {
  my $c = $i - $count + $k +1;
  print OUTPUT "\nColumn $c";
  process_data($final[$k], $missing, $fpfile); 
  }
 
 }


sub process_data
 {
 my @data = @{$_[0]};
 my $missing = $_[1];
 my $fpfile = $_[2];
 
 if ($headerfile)
  {
  if ($headers{$data[0]})
   {
   print "Processing $headers{$data[0]}";
   }
  else
   {
   print "Processing $data[0]";
   }
  }
 else
  {
  print "Processing $data[0]";
  }

 #determine the idcolumn type (numeric, text, coded) 
 my $type = get_type(\@data, $missing, $fpfile);
 print ". Type $type\n";
 
 if ($headerfile)
  {  
  if ($headers{$data[0]})
   {
   print OUTPUT "\nExamining $headers{$data[0]},";    
   }
  else
   {
   print OUTPUT "\nExamining $data[0],"; 
   }
  }
 else
  {
  print OUTPUT "\nExamining $data[0],"; 
  } 
 if ($type =~ /.*CODED$/)
  {
  if ($type eq 'NUMERIC CODED')
   {
   print OUTPUT  " it appears to be a numeric coded set of values\n";
   coded(\@data, $missing);
   if ($plots)
    {
    plot_histogram(\@data, $missing);
    }
   }
  elsif ($type eq 'TEXT CODED')
   {
   print OUTPUT  " it appears to be a text coded set of values\n";
   coded(\@data, $missing);
   } 
  elsif ($type eq 'MIXED CODED')
   {
   print OUTPUT  " it appears to be a mixed coded set of values\n";
   coded(\@data, $missing);
   } 
  }
 elsif ($type eq 'NUMERIC')
  {
  print OUTPUT  " it appears to be a numeric set of values\n";
  numeric(\@data, $missing);
  if ($plots)
   {
   plot_histogram(\@data, $missing);
   }
  }
 elsif ($type eq 'TEXT')
  {
  print OUTPUT  " it appears to be a set of text values\n";
  coded(\@data, $missing);
  }
 elsif ($type eq 'MIXED')
  {
  print OUTPUT  " it appears to be a mixed set of values\n";
  coded(\@data, $missing);
  }
 elsif ($type eq 'MISSING')
  {
  print OUTPUT  " it appears to consist entirely of missing values ($missing)\n";
  }
 else
  {
  
  }
 }

sub coded
 {
 my @data = @{$_[0]};
 my $missingval = $_[1];
 my $missingcount = 0;
 
 my @datavalues = splice(@data, 0, 1); 
 my $total = @data;
 my %count;
 
 print OUTPUT "Found $total values\n";

 for (my $i = 0; $i <= $#data; $i++)
  {
  if ($data[$i] eq $missingval)
   {
   $missingcount++;
   }
  else
   {
   $count{$data[$i]}++;
   }
  }
 
 my @codes = keys %count;
 my $numcodes = @codes;
 
 if ($numcodes == 1)
  {
  print OUTPUT "With $numcodes value:\n";
  }
 else
  {
  print OUTPUT "With $numcodes different values:\n";
  }
 
 if ($missingcount)
  {
  print OUTPUT "Found $missingcount missing values ($missingval)\n"; 
  my $nonmissing = $total - $missingcount;
  print OUTPUT "$nonmissing data values considered as not missing\n";
  }
 
 if ($numcodes < 50 and $numcodes != 0)
  {
  print OUTPUT "Values found are:\n\tCode\tcount\n";
 
  foreach my $code (@codes)
   {
   print OUTPUT "\t$code\t$count{$code}\n";
   }
  }
 elsif ($numcodes > 50) 
  {
  print OUTPUT "more than 50 values found, ($numcodes) first 20 are shown below as an example of column contents\n";
  print OUTPUT "Values found are:\n\tCode\tcount\n";
  
  for (my $i = 0; $i < 20; $i++)
   {
   print OUTPUT "\t$codes[$i]\t$count{$codes[$i]}\n";
   }
  }
 else
  {
  }
 }

sub numeric
 {
 my @rawdata = @{$_[0]};
 my $missingval = $_[1];
 my $j = 0;
 my @data;
 my @datavalues = splice(@rawdata, 0, 1); 
 

 # remove missing values
 for (my $i = 0; $i <= $#rawdata; $i++)
  {
  $rawdata[$i] =~ s/\"//g;
  if ($rawdata[$i])
   {
   if ($rawdata[$i] ne $missing)
    {
    $data[$j] = $rawdata[$i];
    $j++;
    }
   }
  }
 my $total = @rawdata; 
 my $missingcount = $total - $j;
   
 my $average = 0;
 
 my $maxval = max @data;
 my $minval = min @data;
 my $numtotal = @data;
 my $sum = sum @data;
 my $median = median(\@data);
 $median = sprintf("%0.3g", $median); 
 
 
 if ($numtotal != 0)
  {
  $average = sprintf("%0.3g",$sum/$numtotal);
  }

 my $stddev = std_dev(\@data, $average); 
 $stddev = sprintf("%0.3g",$stddev);
 print OUTPUT "Found $total total values\n";
 print OUTPUT "Found $numtotal numeric values\n";
 if ($missingcount)
  {
  print OUTPUT "Found $missingcount missing values ($missingval)\n"; 
  }
 print OUTPUT "Mean:\t\t\t$average\n";
 print OUTPUT "Max Value:\t\t$maxval\n";
 print OUTPUT "Min Value:\t\t$minval\n";
 print OUTPUT "Std Deviation:\t\t$stddev\n";
 print OUTPUT "Median:\t\t\t$median\n";
 print OUTPUT "Checking for outliers +/-3 S.D.\n";
 if ($verbose_flag)
  {
  print LOGFILE "\tVALUE\tGREATER OR LESS THAN 3 SDs\n";  
  } 
 print_outliers(\@data, $average, $stddev, $verbose_flag);
 
 }

sub print_outliers
 {
 my @data = @{$_[0]};
 my $mean = $_[1];
 my $stddev = $_[2];
 my $verbose_flag = $_[3];
 my $limit = 3; # number of std deviations from mean
 my $over = 0;
 my $under = 0;
 my $greater = $mean+($stddev*3);
 my $less = $mean-($stddev*3);
 #print "Greater $greater\nLess $less\n";
 my $sdflag = 0;
  
 for (my $i = 0; $i <= $#data; $i++)
  {
  if ($data[$i])
   {
   if ($data[$i] >= $greater)
    {
    $over++;
    if ($verbose_flag)
     {
     print LOGFILE "\t$data[$i]\tGREATER\n";
     }
    $sdflag = 1;
    }
   elsif ($data[$i] <= $less)
    {
    if ($verbose_flag)
     {
     print LOGFILE "\t$data[$i]\tLESS\n";
     }
    $under++;
    $sdflag = 1;
    }
   } 
  }
   
 if ($sdflag == 0)
  {
  print OUTPUT "...None\n";
  }
 else
  {
  print OUTPUT ">$limit S.D.s\t\t$over\n<$limit S.D.s\t\t$under\n";
  }
 }

sub median 
 {
 my @data = @{$_[0]};
 my @sorted = sort {$a <=> $b} @data; 
 my $median = 0;
 
 if (($#sorted % 2) == 0) # even number of elements
  {
  # median value is the average of the two middle elements
  my $middle = $#sorted/2;
  $median = ($sorted[$middle]+$sorted[$middle-1])/2;
  }
 else # odd number of elements
  {
  $median = $sorted[($#sorted-1)/2];
  }
 return $median;
 }

sub std_dev 
 {
 my @data = @{$_[0]};
 my $mean = $_[1];
 my $sum_x2 = 0;
 my $var = 0;
 my $numvals = @data; 
 my $count = 0;
 my $stddev;
 
 for (my $i = 0; $i < $numvals; $i++)
  {
  if ($data[$i])
   {
   $sum_x2 += (($data[$i] - $mean) ** 2);
   $count++;
   }
  }
  
 if ($count != 0)
  {
  $var = ( $sum_x2 / $count );
  $stddev = sqrt($var);
  }
 else 
  {
  $var = 0;
  $stddev = 0;
  }
  
 return $stddev;
 }

sub get_type
 {
 my @data = @{$_[0]};
 my $missingval = $_[1];
 my $fpfile = $_[2];
 
 my $num = 0;
 my $notnum = 0;
 my $total = 0;
 my %datahash;
 my $type = 'TEXT';
 my @datavalues = splice(@data, 0, 1); 
 my $miss = 0;
 my $totalkeys = 0;
 my $pct = 0;
 
 open FP, ">>$fpfile" or die $!;
 
 for (my $i = 0; $i < $#data; $i++) # check if the value is numeric, 
  {
  if ($data[$i] and $data[$i] ne $missingval) #ensure value is present and isn't the designated missing value id
   {
   my $isnum = looks_like_number($data[$i]);
   if ($isnum)
    {
    $num++;
    }
   else
    {
    $notnum++; 
    }
   $total++; 
   $datahash{$data[$i]}++;
   }
  else
   {
   $miss++;
   }
  } 
 
 if ($total)
  { 
  if ($num == $total) #all numeric
   {
   $type = 'NUMERIC';
   }
  elsif ($notnum == $total) #all non numeric 
   {
   $type = 'TEXT';
   }
  else #mixed numeric/text
   {
   $type = 'MIXED';
   }
  
  $totalkeys = keys %datahash;
  $pct = $totalkeys/$total*100;
  if ($pct < 0.1 or $totalkeys <= 20) # check as a % of total number of values, with hard coded limits for small cohorts
   {
   $type = $type.' CODED';
   }
  }
 else
  {
  $type = 'MISSING';
  }
  
 #prints out a "fingerprint" of the data array for use in future checks
 if ($headerfile)
  {
  if ($headers{$datavalues[0]})
   {
   print FP "$headers{$datavalues[0]}\t$num\t$notnum\t$total\t$totalkeys\t$miss\t$type\t";
   }
  else
   {
   print FP "$datavalues[0]\t$num\t$notnum\t$total\t$totalkeys\t$miss\t$type\t";
   }
  }
 else
  {
  print FP "$datavalues[0]\t$num\t$notnum\t$total\t$totalkeys\t$miss\t$type\t";
  }
 print FP "\t\t\n";
 close FP;
 return $type;  
 }

sub usage
 {
 print "\nUsage:\nperl phenotype-qc.pl -f <phenotypefile> [-v -m <missingvalue> -i <sampleididcolumn> -l <headerfilemapping> -c <columnname> -s <fileofcolumns>]\n";
 print "\n\n";
 printusage("-f --file", "phenotype file", "Tab delimited phenotype file, all rows should contain the same number of columns");
 printusage("-i --idcolumn", "ID column", "Set the column that contains the sample ids, column numbers start from 0, default is 0");
 printusage("-m --missing", "Missing value", "Set the missing value identifier, default is NA");
 printusage("-c --column", "Column", "Specify a single column to check, if using a header file this can be either the human readable or original header");
 printusage("-s --columnfile", "Column file", "Provide a file containing a list of columns to extract, one per row. As with the single column checking can be either header, takes precedence over -c");
 printusage("-l --linker", "Header file", "Provide an optional header file for the data set, for use where the headers within the file are not human readable");
 printusage("-v --verbose", "", "Optional flag to increase verbosity in the log file");
 printusage("-h --help", "", "Show this help message");
 print "\n\n";
 }

sub getwidth
 {
 my $output = `stty size`;
 my @rowcols = split(/\s/, $output);
 my $cols = $rowcols[1];
 if (!$cols)
  {
  $cols = 80;
  }
 return $cols;
 } 
 
sub printusage
 {
 my $option = $_[0];
 my $file = $_[1];
 my $text = $_[2];
 
 my $cols = getwidth(); 

 printf("%-18s", $option);
 printf("%-15s", $file);
 my $textwidth = $cols - 36; #33 plus 3 spaces
 my $newtext = substr ($text, 0, $textwidth);
 
 if (length($text) > $textwidth)
  {
  printf("  %*s", $textwidth, $newtext);
  for (my $i = $textwidth; $i < length($text); $i+= $textwidth)
   {
   my $newtext = substr ($text, $i, $textwidth);
   $newtext =~ s/^\s//;
   printf("%33s", " ");
   printf("   %*s", -$textwidth, $newtext);
   }
  }
 else
  {
  printf("  %*s", -$textwidth, $newtext);
  }
 print "\n"; 
 } 
  
sub get_headers
 {
 my $file = $_[0];
 my %header;
 
 open IN, "$file" or die $!;
 while (<IN>)
  {
  chomp;
  my @temp = split/\t/;
  $header{$temp[0]} = $temp[1];
  }
 close IN;
 return %header;
 }


sub plot_histogram
 {
 my @temp = @{$_[0]};
 my $missing = $_[1];
 my @data;
 my %counts;
 my $header;
 my $missingvals = 0;
 my @clean;
 my $xmax = 0;
 my $xmin = 1000000;
 my $bwidth = 40;
 my $skip;
 my $pwidth = 1800;
 
 if ($headerfile)
  {  
  if ($headers{$temp[0]})
   {
   $header =  $headers{$temp[0]};    
   }
  else
   {
   $header = $temp[0]; 
   }
  }
 else
  {
  $header = $temp[0]; 
  } 
 
 $header =~ s/\//_/g;
 $header =~ s/\\/_/g;
 $header =~ s/\"/_/g;
 $header =~ s/\'/_/g;
 $header =~ s/\:/_/g;
 
 for (@temp[1 .. $#temp])
  {
  if (/$missing/)
   {
   $missingvals++;
   }
  else
   {
   if ($_ > 0)
    {
    $_ = int($_+0.5);
    }
   else
    {
    $_ = int($_-0.5);
    }
   push (@clean, $_);
   if ($_ > $xmax) # set x max value
    {
    $xmax = $_;
    }
   if ($_ < $xmin) #set x min 
    {
    $xmin = $_;
    }
   }
  }
 
 for (my $i = 1; $i <= $#clean; $i++)
  {
  $counts{$clean[$i]}++;
  }
 
 if ($xmax - $xmin > 800)
  {
  }
 else
  {
  for ($xmin .. $xmax) #pad with all values from min to max to plot correctly.
   {
   if (!$counts{$_})
    {
    $counts{$_} = 0;
    }
   }
  }
  
 my @ktemp = keys %counts;
 my @kdata = sort {$a <=> $b} @ktemp;
 
 for (my $i = 0; $i <= $#kdata; $i++)
  {
  $data[0][$i] = $kdata[$i];
  $data[1][$i] = $counts{$kdata[$i]}; 
  }
 
 my $outfile = $header.'.png';
 
 print "Writing plot to $outfile\n";
  
 if ($#kdata > 0)
  {
  $bwidth = int(1200/$#kdata);
  }
# print "$bwidth\n";
 
 if ($bwidth == 0) #if column width == 0 set to 1
  {
  $bwidth = 1;
  } 
 
 if ($#kdata > 1200) # if more than 1200 bars then resize the plot to accomodate
  {
  $pwidth = $#kdata+300;
  }
 
 if ($#kdata < 25)
  {
  $skip = 1;
  }
 elsif ($#kdata < 200)
  {
  $skip = 5;
  }
 elsif ($#kdata < 500)
  {
  $skip = 10;
  }
 else
  {
  $skip = 50
  }
  
 my $graph = new GD::Graph::bars($pwidth,1200);
  
 $graph->set_title_font('14459.ttf', 18);
 $graph->set_x_label_font('14459.ttf', 18);
 $graph->set_y_label_font('14459.ttf', 18);
 $graph->set_x_axis_font('14459.ttf', 14);
 $graph->set_y_axis_font('14459.ttf', 14);
 $graph->set_legend_font('14459.ttf', 14);
 $missingvals = 'Missing: '.$missingvals;
 $missingvals = 'Missing: '.$missingvals.' MAX: '.$xmax.' MIN: '.$xmin;
 
 $graph->set
  ( 
  x_label           => $header,
  y_label           => 'Count',
  title             => "Histogram of $header",
  x_ticks           => 1,
  x_label_skip      => $skip,
  x_last_label_skip => 1,
  bar_width         => $bwidth,
  tick_length       => -5,
  x_labels_vertical => 1,
  t_margin          => 20,
  b_margin          => 40,
  l_margin          => 10,
  r_margin          => 40,  
  dclrs             => [ qw(dred orange) ],
  labelclr          => 'black',
  axislabelclr      => 'black',
  legendclr         => 'black',
  textclr           => 'black',
  fgclr             => 'black',
  transparent       => 0,
  #legend_placement  => 'RT',
  legend_marker_width => 0,
  legend_marker_height => 0
  ) 
 or warn $graph->error;
 
 if (@data)
  {
  my $gd = $graph->plot(\@data) or die $graph->error;
          
  open(IMG, ">$outfile") or die $!;
  binmode IMG;
  print IMG $gd->png;
  close IMG;
  }
 }
 
